@@ -13,8 +13,6 @@ from activitysim.core import inject
 from activitysim.core import pipeline
 from activitysim.core import expressions
 
-from activitysim.core.mem import force_garbage_collect
-
 from activitysim.core import assign
 from activitysim.core import los
 
@@ -160,7 +158,7 @@ def trip_mode_choice(
             chunk_size=chunk_size,
             mode_column_name=mode_column_name,
             logsum_column_name=logsum_column_name,
-            trace_label=trace_label,
+            trace_label=segment_trace_label,
             trace_choice_name='trip_mode_choice',
             estimator=estimator)
 
@@ -182,9 +180,6 @@ def trip_mode_choice(
 
         choices_list.append(choices)
 
-        # FIXME - force garbage collection
-        force_garbage_collect()
-
     choices_df = pd.concat(choices_list)
 
     # add cached tvpb_logsum tap choices for modes specified in tvpb_mode_path_types
